<?php

namespace ByDedi\AdminBundle\Annotation;

use Doctrine\Common\Annotations\AnnotationReader;

class UploadableAnnotationReader
{

    /**
     * @var AnnotationReader
     */
    private $reader;

    /**
     * @var \ReflectionProperty[][]
     */
    private static $uploadableProperties = [];

    /**
     * UploadableAnnotationReader constructor.
     * @param AnnotationReader $reader
     */
    public function __construct(AnnotationReader $reader)
    {
        $this->reader = $reader;
    }

    /**
     * @param $entity
     * @return array
     */
    public function getUploadableFields($entity)
    {
        $properties = $this->getEntityUploadableFields($entity);
        return $properties;
    }

    private function getEntityUploadableFields($entity)
    {
        $class = get_class($entity);
        $reflection = new \ReflectionClass($class);
        $annotations = [];
        if (!array_key_exists($class, self::$uploadableProperties)) {
            $properties = $reflection->getProperties();
        } else {
            $properties = self::$uploadableProperties[$class];
        }

        self::$uploadableProperties += [$class => []];

        foreach ($properties as $property) {
            $annotation = $this->reader->getPropertyAnnotation($property, Uploadable::class);
            if ($annotation !== null) {
                $annotations[$property->getName()] = $annotation;
                self::$uploadableProperties[$class] += [$property->getName() => $property];
            }
        }

        return $annotations;
    }
}
